@@ -138,6 +138,8 @@ static gboolean app_device_autorun      (DevInfo *dev,
                                          AppInfo *ai);
 static void     app_device_ask_mixed    (DevInfo *dev,
                                          AppInfo *ai);
+static void     app_device_activate_cdrom (DevInfo *dev,
+		                         AppInfo *ai);
 
 
 static gboolean mntent_has_option (const char *optlist, const char *option);
@@ -397,7 +399,8 @@ app_close (AppInfo *ai)
                         argv[0] = "/bin/umount";
                         argv[1] = dev->fspath;
                         argv[2] = NULL;
-                        gnome_execute_async (g_get_home_dir (), 2, argv);
+			g_spawn_sync (g_get_home_dir (), argv, NULL, 0,
+					NULL, NULL, NULL, NULL, NULL, NULL);
                 }
         }
 }
@@ -556,8 +559,11 @@ app_check_devices (AppInfo *ai,
 #endif /* MD_DEBUG */
         }
 
-        if (initial)
+        if (initial) {
 		gtk_timeout_add (CHECK_INTERVAL, (GtkFunction)app_devices_check_change, ai);
+		/* Try to mount all the devices on startup */
+		g_list_foreach (ai->devices, (GFunc)app_device_activate_cdrom, ai);
+	}
 
         return changed;
 }
@@ -566,10 +572,16 @@ static gboolean
 mntent_has_fs_type (struct mntent *ent,
                     const gchar   *fstype)
 {
-        char **fstypes = g_strsplit (ent->mnt_type, ",", -1);
+        char **fstypes;
         int i;
-        gboolean result = TRUE;
-        
+        gboolean result = FALSE;
+
+	if (strcmp (ent->mnt_type, "auto") == 0) {
+		return TRUE;
+	}
+
+	fstypes = g_strsplit (ent->mnt_type, ",", -1);
+
         for (i = 0; fstypes[i]; i++) {
                 if (strcmp (fstypes[i], fstype) == 0) {
                         result = TRUE;
@@ -577,9 +589,9 @@ mntent_has_fs_type (struct mntent *ent,
                 }
         }
 
-        g_strfreev (fstypes);
+	g_strfreev (fstypes);
 
-        return result;
+	return result;
 }
 
 static gboolean
@@ -832,8 +844,10 @@ static gboolean
 app_device_floppy_add (AppInfo *ai,
                        DevInfo *dev)
 {
-        if (my_g_check_permissions (dev->fsname, R_OK))
+        if (my_g_check_permissions (dev->fsname, R_OK)) {
+		magicdev_verbose ("Floppy drive '%s' can't be monitored because we don't have access rights to it", dev->fsname);
                 return FALSE;
+	}
 
         dev->type = DEV_FLOPPY;
 
@@ -909,9 +923,19 @@ static gboolean
 app_device_iso9660_add (AppInfo *ai,
                         DevInfo *dev)
 {
-        dev->fd = open (dev->fsname, O_RDONLY|O_NONBLOCK|O_EXCL);
-        if (dev->fd < 0)
-                return FALSE;
+	if (dev->fd < 0) {
+		dev->fd = open (dev->fsname, O_RDONLY|O_NONBLOCK|O_EXCL);
+	}
+
+        if (dev->fd < 0) {
+		if (errno != EBUSY) {
+			return FALSE;
+		} else {
+			/* Assume that if it's busy, that's because it's
+			 * an O_EXCL'ed CD drive */
+			return TRUE;
+		}
+	}
 
         dev->type = DEV_CDROM;
 
@@ -964,7 +988,7 @@ app_device_cdrom_set_state (DevInfo *dev,
         }
 
 	if (dev->fd < 0) {
-		return;
+		goto end;
 	}
 
 	if ((dev->supports_changed == TRUE) &&
@@ -992,14 +1016,15 @@ app_device_cdrom_set_state (DevInfo *dev,
                 default:
                         dev->state = STATE_EMPTY;
                         break;
-                }
-        } else
+		}
+        } else {
                 dev->state = STATE_EMPTY;
+	}
 
 end:
         if (app_device_is_mounted (dev))
                 dev->state = STATE_ACTIVE;
-        
+
 	close (dev->fd);
 	dev->fd = -1;
 }
@@ -1035,6 +1060,9 @@ app_device_activate_cdrom (DevInfo *dev,
 	}
 
         disctype = ioctl (dev->fd, CDROM_DISC_STATUS, CDSL_CURRENT);
+	close (dev->fd);
+	dev->fd = -1;
+
         switch (disctype) {
         case CDS_AUDIO:
                 app_device_run_cdplayer (dev, ai);
@@ -1060,9 +1088,6 @@ app_device_activate_cdrom (DevInfo *dev,
                 /* Huh? */
                 break;
         }
-
-	close (dev->fd);
-	dev->fd = -1;
 }
 
 static void
@@ -1139,10 +1164,6 @@ app_device_check_change (DevInfo *dev,
 #endif
 		if (func)
 			func (dev, ai);
-#if 0
-                if (func)
-                        ai->needs_desktop_refresh |= func (dev, ai);
-#endif
         }
 }
 
@@ -1170,7 +1191,7 @@ app_devices_update_is_mounted (AppInfo *ai)
         GList *ltmp;
         DevInfo *dev;
         struct stat statbuf;
-        
+
         if (stat (_PATH_MOUNTED, &statbuf) == 0) {
                 time_t new_time = MAX (statbuf.st_mtime, statbuf.st_ctime);
 
@@ -1235,7 +1256,8 @@ app_device_run_command (DevInfo *dev, char *orig_command)
 	argv[i++] = exec_str->str;
 	argv[i] = NULL;
 
-	gnome_execute_async (g_get_home_dir (), i, argv);
+	g_spawn_async (g_get_home_dir (),
+			argv, NULL, 0, NULL, NULL, NULL, NULL);
 
 	g_string_free (exec_str, TRUE);
 	g_free (command);
@@ -1274,12 +1296,14 @@ app_device_mount (DevInfo *dev,
                   AppInfo *ai)
 {
         char *argv[3];
-        
+
         if (ai->do_automount == TRUE) {
                 argv[0] = "/bin/mount";
                 argv[1] = dev->fspath;
                 argv[2] = NULL;
-		gnome_execute_async (g_get_home_dir (), 2, argv);
+
+		g_spawn_sync (g_get_home_dir (), argv, NULL, 0,
+				NULL, NULL, NULL, NULL, NULL, NULL);
                 dev->did_mount = TRUE;
         }
 }
@@ -1316,11 +1340,11 @@ app_device_autorun (DevInfo *dev,
 {
         char *check_path;
         char *argv[2];
-        int pid = -1;
         gboolean do_quit = FALSE;
 
 	if (is_video_dvd (dev) && ai->do_dvd_play) {
 		app_device_run_dvdplayer (dev, ai);
+		return TRUE;
 	}
 
         if (ai->do_autorun == TRUE) {
@@ -1328,10 +1352,9 @@ app_device_autorun (DevInfo *dev,
                 int i;
 
                 app_freeze (ai);
-
                 autorun_fns = g_strsplit (ai->autorun_path, ":", -1);
 
-                for (i = 0; autorun_fns[i] && pid < 0 && !do_quit; i++) {
+                for (i = 0; autorun_fns[i] && !do_quit; i++) {
 
                         check_path = g_strdup_printf ("%s/%s",
 					dev->fspath, autorun_fns[i]);
@@ -1339,10 +1362,14 @@ app_device_autorun (DevInfo *dev,
 			argv[1] = NULL;
 
                         if (!my_g_check_permissions (check_path, X_OK))
+			{
+				magicdev_verbose ("CD-Rom drive '%s' can't be monitored because we don't have access rights to it", check_path);
                                 continue;
+			}
 
 			if (run_me (check_path)) {
-				pid = gnome_execute_async (g_get_home_dir (), 1, argv);
+				g_spawn_async (dev->fspath, argv, NULL,
+						0, NULL, NULL, NULL, NULL);
 				do_quit = TRUE;
 			}
 
@@ -1389,7 +1416,7 @@ app_device_ask_mixed (DevInfo *dev,
         } else if (ai->do_cd_play) {
                 action = PLAY;
         }
-                
+
         switch (action) {
         case MOUNT:
                 app_device_mount (dev, ai);
